import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, TicksPerSecond } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.2.0";
export const VERSION_HISTORY = ["1.2.0", "1.1.0"];

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_tpa:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_tpa:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});








world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_tpa_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_tpa_version", VERSION);
    if (player.hasTag("sg_tpa_received_guide")) {
      player.removeTag("sg_tpa_received_guide");
      player.sendMessage({
        translate: "sg_tpa.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_tpa:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_tpa_version", VERSION);
    player.sendMessage({
      translate: "sg_tpa.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // TPA
    translation_key: "sg_tpa.guide.teleport.title",
    icon_path: "textures/ui/world_glyph_color.png",
    action: (player) => {
      // Create and show the modal form
      const form = new ActionFormData();
      form.title({
        translate: "sg_tpa.guide.teleport.title",
      });
      form.body("Which player would you like to ask to teleport to?\n\n");

      const players = world.getAllPlayers(); // Get all players
      for (const targetPlayer of players) {
        form.button(targetPlayer.name); // Add a button for each player
      }

      form.show(player).then((res) => {
        if (res.canceled || res.selection < 0) return; // Ensure the selection is valid

        const selectedPlayer = players[res.selection]; // Get the selected player based on index
        if (selectedPlayer.name == player.name) {
          player.sendMessage("§cYou cannot teleport to yourself.");
          player.playSound("note.bit", { location: player.location, pitch: 0.5, volume: 1 });
          return;
        }
        player.sendMessage("§eYour teleport request was sent...");
        player.playSound("mob.phantom.flap", { location: player.location, pitch: 2, volume: 1 });
        player.playSound("mob.phantom.flap", { location: player.location, pitch: 1.8, volume: 1 });
        selectedPlayer.playSound("note.bit", { location: selectedPlayer.location, pitch: 1.88, volume: 1 });

        // Create and show the teleport request form to the selected player
        const teleportForm = new ActionFormData();
        teleportForm.title("Teleport Request");
        teleportForm.body(player.name + " is requesting to teleport to you.\n");
        teleportForm.button("Accept", "textures/ui/confirm.png");
        teleportForm.button("Decline", "textures/ui/cancel.png");

        teleportForm.show(selectedPlayer).then((response) => {
          if (response.selection === 0) {
            // Handle the "Accept" response here (e.g., teleport the requesting player)
            selectedPlayer.runCommand(`gamerule sendCommandFeedback false`);
            selectedPlayer.runCommand(`tp ${player.name} ~ ~ ~ ~ ~`);
            selectedPlayer.runCommand(`gamerule sendCommandFeedback true`);
            player.sendMessage("§aYour teleport request was accepted.");
            selectedPlayer.sendMessage("§aYou accepted the teleport request.");
            player.playSound("note.pling", { location: player.location, pitch: 1.88, volume: 1 });
            selectedPlayer.playSound("note.pling", { location: selectedPlayer.location, pitch: 1.88, volume: 1 });
          } else {
            // Handle the "Decline" response here
            player.sendMessage("§cYour teleport request was declined.");
            selectedPlayer.sendMessage("§cYou declined the teleport request.");
            player.playSound("note.bit", { location: player.location, pitch: 0.5, volume: 1 });
            selectedPlayer.playSound("note.bit", { location: selectedPlayer.location, pitch: 0.75, volume: 1 });
          }
        });
      });
    },
  },

  {
    // TPR
    translation_key: "sg_tpa.guide.tpr.title",
    icon_path: "textures/ui/realmsIcon.png",
    action: (player) => {
      // Create and show the TPR modal form
      const form = new ModalFormData()
        .title("Random Teleport")
        .slider("Select maximum distance (blocks)", 10, 1000, 10, 100); // Example slider values

      form.show(player).then((res) => {
        if (res.canceled) return;

        const maxDistance = res.formValues[0]; // Get the selected max distance

        // Run the spreadplayers command
        player.runCommand(`spreadplayers ~ ~ 0 ${maxDistance} @s`);
        if (maxDistance>100) {
          player.runCommand("camera @s fade time 0 3.3 1 color 0 0 0")
        } else {
          player.runCommand("camera @s fade time 0 0.5 0.5 color 0 0 0");
        }
        player.runCommand(`tp ~ ~0.1 ~`);
        const delay = 1; // Define delay between commands

        const commands = [
          `execute if blocks ~ ~ ~ ~ ~-128 ~ ~ 170 ~ all run tp ~ ~-128 ~`,
          `execute if blocks ~ ~ ~ ~ ~-64 ~ ~ 230 ~ all run tp ~ ~-64 ~`,
          `execute if blocks ~ ~ ~ ~ ~-32 ~ ~ 270 ~ all run tp ~ ~-32 ~`,
          `execute if blocks ~ ~ ~ ~ ~-32 ~ ~ 270 ~ all run tp ~ ~-32 ~`,
          `execute if blocks ~ ~ ~ ~ ~-32 ~ ~ 270 ~ all run tp ~ ~-32 ~`,
          `execute if blocks ~ ~ ~ ~ ~-16 ~ ~ 290 ~ all run tp ~ ~-16 ~`,
          `execute if blocks ~ ~ ~ ~ ~-8 ~ ~ 300 ~ all run tp ~ ~-8 ~`,
          `execute if blocks ~ ~ ~ ~ ~-4 ~ ~ 300 ~ all run tp ~ ~-4 ~`,
          `execute if blocks ~ ~ ~ ~ ~-2 ~ ~ 300 ~ all run tp ~ ~-2 ~`,
          `execute if blocks ~ ~ ~ ~ ~-1 ~ ~ 300 ~ all run tp ~ ~-1 ~`,
        ];

        // Function to run commands in succession
        function runCommandsSequentially(index = 0) {
          if (index < commands.length) {
            player.runCommand(commands[index]);
            // Run the next command after a delay
            system.runTimeout(() => runCommandsSequentially(index + 1), delay);
          }
        }

        // Start running commands
        system.runTimeout(() => runCommandsSequentially(), 3 * TicksPerSecond);

        player.sendMessage(`§aYou are being randomly teleported within ${maxDistance} blocks...`);
        player.playSound("note.pling", { location: player.location, pitch: 1.5, volume: 1 });
      });
    },
  },

  {
    // CHANGELOG
    translation_key: "sg_tpa.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_tpa.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_tpa.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_tpa.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_tpa.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_tpa.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_tpa.guide.common.exit_button",
  },
];


const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_tpa:guide") {
      showMainPage(event.source);
    }
  });
});